@@ -26,6 +26,7 @@
  **********************************************************************/
 
 #include "vanessa_socket.h"
+#include "unused.h"
 
 #include <errno.h>
 
@@ -46,7 +47,7 @@
  **********************************************************************/
 
 ssize_t vanessa_socket_pipe_fd_read(int fd, void *buf, size_t count,
-				    void *data)
+				    void *UNUSED(data))
 {
 	ssize_t bytes;
 
@@ -79,7 +80,7 @@ ssize_t vanessa_socket_pipe_fd_read(int fd, void *buf, size_t count,
  **********************************************************************/
 
 ssize_t vanessa_socket_pipe_fd_write(int fd, const void *buf, size_t count,
-				     void *data)
+				     void *UNUSED(data))
 {
 	size_t bytes;
 
@@ -130,14 +131,14 @@ ssize_t vanessa_socket_pipe_fd_write(int fd, const void *buf, size_t count,
 
 static int __vanessa_socket_pipe_dummy_select(int n, fd_set *readfds, 
 		fd_set *writefds, fd_set *exceptfds, struct timeval *timeout, 
-		void *data)
+		void *UNUSED(data))
 {
 	return(select(n, readfds, writefds, exceptfds, timeout));
 }
 
 
 int vanessa_socket_pipe_func(int rfd_a, int wfd_a, int rfd_b, int wfd_b, 
-		unsigned char *buffer, int buffer_length, int idle_timeout, 
+		char *buffer, int buffer_length, int idle_timeout,
 		size_t *return_a_read_bytes, size_t *return_b_read_bytes, 
 		ssize_t(*read_func) (int fd, void *buf, size_t count, 
 			void *data), 
@@ -243,7 +244,7 @@ int vanessa_socket_pipe_func(int rfd_a, int wfd_a, int rfd_b, int wfd_b,
  **********************************************************************/
 
 ssize_t vanessa_socket_pipe_read_write_func(int rfd, int wfd, 
-		unsigned char *buffer, int buffer_length, 
+		char *buffer, int buffer_length,
 		ssize_t(*read_func) (int fd, void *buf, size_t count, 
 			void *data), 
 		ssize_t(*write_func) (int fd, const void *buf, size_t count, 
@@ -297,7 +298,7 @@ ssize_t vanessa_socket_pipe_read_write_func(int rfd, int wfd,
  *         0 otherwise
  **********************************************************************/
 
-int vanessa_socket_pipe_write_bytes_func(int fd, const unsigned char *buffer, 
+int vanessa_socket_pipe_write_bytes_func(int fd, const char *buffer,
 		const ssize_t n, 
 		ssize_t(*write_func) (int fd, const void *buf, size_t count, 
 			void *data), 
